/*
 * EXIF date tool
 *
 * (c) Alexander Vdolainen 2023 <alex@vapaa.xyz>
 *
 * this is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * this is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.";
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <string.h>
#include <time.h>
#include <utime.h>

#include <libexif/exif-data.h>

#include <config.h>

#ifndef TOOL_FULLNAME
#define TOOL_FULLNAME  "Jpeg EXIF data date tool"
#endif

typedef enum {
  EDT_MODIFY_FO = 0,
  EDT_DISPLAY = 1,
  EDT_MODIFY_ED = 2,
} edt_mode_t;

static void display_help(FILE *o, const char *exec_name)
{
  fprintf(o, "%s: %s\n\n", exec_name, TOOL_FULLNAME);
  /* usage options */
  fprintf(o, "Usage:\n");
  fprintf(o, "\t%s\n", exec_name);
  fprintf(o, "\t%s <jpeg filename>\n", exec_name);

  fprintf(o, "\nOptions:\n");
  fprintf(o, "\t%-36s Display date and time from exif data only, don't update file creation date.\n",
          "-d | --display-date");
  fprintf(o, "\t%-36s Fix/create EXIF creation date based on the file creation date.\n",
          "-e | --exif-date-fix");
  fprintf(o, "\t%-36s Display help.\n", "-h | --help");
  fprintf(o, "\t%-36s Display version information.\n\n", "-v | --version");

  return;
}

extern time_t exifdt_tm(const char *);

int main(int argc, char **argv)
{
  int opt, fargs = 0, i = 0;
  edt_mode_t mode = EDT_MODIFY_FO;
  char **fn_name = NULL;
  ExifData *ef;
  ExifEntry *entry;
  char dtbuf[1024];
  time_t exif_dt;
  struct utimbuf ntimes;

  while(1) {
    int option_index = 0;
    static struct option long_options[] =
    {
      {"display-date", no_argument, NULL, 'd'},
      {"exif-date-fix", no_argument, NULL, 'e'},
      {"help", no_argument, NULL, 'h'},
      {"version", no_argument, NULL, 'v'},
      {NULL, 0, NULL, 0},
    };

    if((opt = getopt_long(argc, argv, "dehv", long_options, &option_index)) == -1)
      break;

    switch(opt) {
      case 'd':
        mode = EDT_DISPLAY;
        break;
      case 'e':
        mode = EDT_MODIFY_ED;
        break;
      case 'h':
        display_help(stdout, argv[0]);
        return 0;
        break;
      case 'v':
        fprintf(stdout, "%s: %s v.: %s\n", argv[0], TOOL_FULLNAME, VERSION);
        return 0;
        break;
    }
  }

  /* pick up other arguments */
  if(optind < argc) {
    fargs = argc - optind;
    if(!(fn_name = malloc(sizeof(uintptr_t)*(fargs + 1)))) {
      __enomem:
      fprintf(stderr, "Not enough memory.\n");
      return -1;
    } else memset(fn_name, 0, sizeof(uintptr_t)*(fargs + 1));

    while(optind < argc) {
      if(!(fn_name[i] = strdup((const char *)argv[optind++]))) {
        for(; i >= 0; i--) {
          if(fn_name[i]) free(fn_name[i]);
        }
        free(fn_name);
        goto __enomem;
      } else i++;
    }
  } else {
    /* no files has been provided */
    fprintf(stderr, "Missing filename(s) argument.\n");
    display_help(stdout, argv[0]);
    return 1;
  }

  /* got thru the list */
  i = 0;
  while(fn_name[i] != NULL) {
    ef = exif_data_new_from_file(fn_name[i]);
    if(!ef) fprintf(stderr, "File not found or no EXIF data exists.\n");
    else {
      /* read date and time */
      entry = exif_content_get_entry(ef->ifd[EXIF_IFD_EXIF], EXIF_TAG_DATE_TIME_ORIGINAL);
      if(entry) {
        exif_entry_get_value(entry, dtbuf, sizeof(dtbuf));
        if(*dtbuf) {
          switch(mode) {
            case EDT_DISPLAY:
              fprintf(stdout, "%s date and time: %s\n",fn_name[i], dtbuf);
              break;
            case EDT_MODIFY_FO:
              /* convert exif format to timeval */
              exif_dt = exifdt_tm(dtbuf);
              if(exif_dt == (time_t) -1)
                fprintf(stderr, "%s: Date and time data in exif is invalid.\n", fn_name[i]);
              else { /* modify mtime */
                ntimes.actime = exif_dt;
                ntimes.modtime = exif_dt;
                utime(fn_name[i], &ntimes);
              }
              break;
            case EDT_MODIFY_ED:
              fprintf(stdout, "modify\n");
              break;
          }
        } else fprintf(stdout, "%s: No date and time information exists in exif.\n",
                       fn_name[i]);
      } else {
        switch(mode) {
          case EDT_DISPLAY:
          case EDT_MODIFY_FO:
            fprintf(stdout, "%s: No time date information found in EXIF.\n",
                    fn_name[i]);
            break;
          case EDT_MODIFY_ED:
            fprintf(stdout, "modify\n");
            break;
        }
      }
      exif_data_unref(ef);
    }
    free(fn_name[i]);
    i++;
  }
  free(fn_name);

  return 0;
}
